#!/usr/bin/perl

use POSIX qw(strftime sprintf);
use Time::Local;

my $PROC_ALARM = "/proc/acpi/alarm";
my $SETTIME = "/usr/bin/settime.sh";
my $DEBUG = 0;
my $WAKEUP_MARGIN = 5;
my $TWOOCLOCK = 1;
my $METHOD = 1; # 0 = ACPI-Wakeup, 1 = settime, 2 = autodetect

sub true() { 1; }
sub false() { 0; }

sub dprint {
  $_ = join("", @_);
  chomp;
  print "$_\n" if($DEBUG);
}

sub setTime {
  system("hwclock -w");
  #system("hwclock -w --directisa");
}

sub setAlarm {
  my $Next = shift;
  my $TwoOclock = NextTwoOclock();
  my $NextStart;
  my $ShouldStart;
  setTime();

  if($Next) {
    if($Next > $TwoOclock) {
      $ShouldStart = true;
    }
    $NextStart = $Next - ($WAKEUP_MARGIN * 60);
    dprint("Next event at ", strftime("%d.%m.%Y %H:%M", localtime($Next)));
  } else {
    $ShouldStart = true;
  }

  if($TWOOCLOCK && $ShouldStart) {
    $NextStart = $TwoOclock;
  }

  if($NextStart) {
    dprint("Programming wakeup at ", strftime("%d.%m.%Y %H:%M:%S", localtime($NextStart)));

    if($METHOD == 2) {
      if(-e $PROC_ALARM) {
        $METHOD = 0;
      } else {
        $METHOD = 1;
      }
    }

    if($METHOD == 0) {
      if(-e $PROC_ALARM) {
        system(sprintf("echo \"%s\" > %s", strftime("%Y-%m-%d %H:%M:%S", localtime($NextStart)), $PROC_ALARM));
        sleep 1;
        system(sprintf("echo \"%s\" > %s", strftime("%Y-%m-%d %H:%M:%S", localtime($NextStart)), $PROC_ALARM));
      } else {
        dprint("$PROC_ALARM does not exist");
      }
    } else {
      if(-e $SETTIME) {
        system(sprintf("%s \"%s\"", $SETTIME, strftime("%Y-%m-%d %H:%M:00", localtime($NextStart))));
      } else {
        dprint("$SETTIME does not exist");
      }
    }
  }
}

sub NextTwoOclock {
  my $now = time();
  # Array-Format: ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
  my @today = localtime($now);
  @today[0..2] = (0, 0, 2);

  # get today 2:00 in seconds since epoc
  my $TwoOclock = timelocal(@today);

  # Check: Is today 2:00 in future or past?
  if($now < $TwoOclock) {
    # Today two oclock is in future
    return $TwoOclock;
  } else {
    # We're past two oclock, next two oclock is tomorrow (+86400s)
    return $TwoOclock+86400;
  }
}

open(in, "</etc/alarm") || die "Alarm file not present\n";
@alarm = <in>;
close in;
die "Alarm file empty\n" if(scalar(@alarm) == 0);
my($Next) = @alarm;
setAlarm($Next);
print $Next;
unlink('/etc/alarm');
